@@ -1,18 +1,16 @@
 # -*- coding: utf-8 -*-
 # -- By TRAKOS -- #
-import json
-import logging
-import os
-import time
-import requests
-import json 
+import json, logging, os, time, requests, json , time
+from rebot import regular
+from datetime import datetime
 from collections import namedtuple
 print(f"Logger Started - Rebot - api.telegram.org")
 logger = logging.getLogger(__name__)
 import json
 from requests import get,post
 import requests
-
+import itertools
+from threading import Thread
 BASE_URL = "https://api.telegram.org/bot" 
 class dotdict(dict):
     __getattr__= dict.get
@@ -25,9 +23,10 @@ class Bot:
         self.token = token
         self.url = BASE_URL
         self.marker = None
+        
     def to_json(self):
         return json.dumps(self.to_dict())
-    def get_updates(self, limit=None, timeout=80):
+    def get_updates(self, limit=None, timeout=10):
         
         update = {}
         non = []
@@ -37,7 +36,7 @@ class Bot:
             exit()
         else:
             try:
-                response = requests.get(f"https://api.telegram.org/bot{self.token}/getUpdates", timeout=100)
+                response = requests.get(f"https://api.telegram.org/bot{self.token}/getUpdates", timeout=90)
                 update = response.json()
             except requests.exceptions.ReadTimeout:
                 logger.info('get_updates ReadTimeout')
@@ -59,27 +58,33 @@ class Bot:
             if update:
                 a = 0
             a = update['result'][-1]
+            
             import re
             
+            if "reply_to_message" in str(a):
+                aae = {"upid":a['update_id'],"userid":a['message']['from']['id'],"message_id":a['message']['message_id'],"text":a['message']['text'],"fname":a['message']['from']['first_name'],"chat_id":a['message']['chat']['id'],"chat_type":a['message']['chat']['type'],"reply_message_id":a['message']['reply_to_message']['message_id'],"reply_to_userid":a['message']['reply_to_message']['from']['id'],"reply_to_isbot":a['message']['reply_to_message']['from']['is_bot'],"reply_to_fname":a['message']['reply_to_message']['from']['first_name'],"reply_to_text":a['message']['reply_to_message']['text']}
+                dict = dotdict(aae)
+                return dict
             if "callback_query" in str(a) or "reply_markup" in str(a):
-                ae = {"upid":a['update_id'],"data":a['callback_query']['data'],"chat_id":a['callback_query']['message']['chat']['id'],"userid":a['callback_query']['from']['id'],"text":a['callback_query']['message']['text'],"message_id":a['callback_query']['message']['message_id']}
+                ae = {"upid":a['update_id'],"call_id":a['callback_query']['id'],"data":a['callback_query']['data'],"chat_id":a['callback_query']['message']['chat']['id'],"userid":a['callback_query']['from']['id'],"text":a['callback_query']['message']['text'],"message_id":a['callback_query']['message']['message_id']}
                 dict = dotdict(ae)
                 return dict
             if "set_name" in str(a):
                 set = (re.findall("'set_name': '(.*?)'",str(a))[0])
                 emo = (re.findall("'emoji': '(.*?)'",str(a))[0])
-                aa = {"upid":a['update_id'],"message_id":a['message']['message_id'],"userid":a['message']['from']['id'],"Username":a['message']['from']['username'],"fname":a['message']['from']['first_name'],"chat_id":a['message']['chat']['id'],"chat_type":a['message']['chat']['type'],"sticker":emo,"text":set}
+                aa = {"upid":a['update_id'],"message_id":a['message']['message_id'],"userid":a['message']['from']['id'],"fname":a['message']['from']['first_name'],"chat_id":a['message']['chat']['id'],"chat_type":a['message']['chat']['type'],"sticker":emo,"text":set}
                 dict = dotdict(aa)
                 return dict
             if "dice" in str(a):
-                aa = {"upid":a['update_id'],"message_id":a['message']['message_id'],"userid":a['message']['from']['id'],"Username":a['message']['from']['username'],"fname":a['message']['from']['first_name'],"chat_id":a['message']['chat']['id'],"chat_type":a['message']['chat']['type'],"text":a['message']['dice']['emoji']}
+                aa = {"upid":a['update_id'],"message_id":a['message']['message_id'],"userid":a['message']['from']['id'],"fname":a['message']['from']['first_name'],"chat_id":a['message']['chat']['id'],"chat_type":a['message']['chat']['type'],"text":a['message']['dice']['emoji']}
                 dict = dotdict(aa)
                 return dict
             if "text" in str(a):
                 
-                aa = {"upid":a['update_id'],"message_id":a['message']['message_id'],"userid":a['message']['from']['id'],"Username":a['message']['from']['username'],"fname":a['message']['from']['first_name'],"chat_id":a['message']['chat']['id'],"chat_type":a['message']['chat']['type'],"text":a['message']['text']}
+                aa = {"upid":a['update_id'],"message_id":a['message']['message_id'],"userid":a['message']['from']['id'],"fname":a['message']['from']['first_name'],"chat_id":a['message']['chat']['id'],"chat_type":a['message']['chat']['type'],"text":a['message']['text']}
                 dict = dotdict(aa)
                 return dict
+    
     def file(self,chat_id=None,file=None,reply_to_message_id=None, reply_markup=None,
         parse_mode=None):
         files = {'document': open(file, 'rb')}
@@ -301,9 +306,24 @@ class Bot:
             aa = {"badRequest":True,"message":des}
             dict = dotdict(aa)
             return dict
-    def photos(self,chat_id,limit=None):
-        r = requests.get("https://api.telegram.org/bot{}/getUserProfilePhotos?chat_id={}&limit={}".format(self.token,chat_id,limit))
-        return r.text
+    def photos(self, user_id, offset=None, limit=None):
+        
+        dataset = {'user_id': user_id}
+        if offset:
+            dataset['offset'] = offset
+        if limit:
+            dataset['limit'] = limit
+        r = requests.get("https://api.telegram.org/bot{}/getUserProfilePhotos".format(self.token), params=dataset)
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    
     def photo(self,chat_id,photo,caption=None,parse_mode=None,disable_notification=None,protect_content=None,reply_to_message_id=None,allow_sending_without_reply=None,reply_markup=None):
         if reply_markup:
             if "https" not in str(photo) or "http" not in str(photo):
@@ -810,6 +830,97 @@ class Bot:
             aa = {"badRequest":True,"message":des}
             dict = dotdict(aa)
             return dict
+    def convert_input_media_array(array):
+        media = []
+        files = {}
+        
+        for input_media in array:
+            if isinstance(input_media, regular.InputMedia):
+                media_dict = input_media.to_dict()
+                if media_dict['media'].startswith('attach://'):
+                    key = media_dict['media'].replace('attach://', '')
+                    files[key] = input_media.media
+                media.append(media_dict)
+        return json.dumps(media), files
+    def media_group(
+        self, chat_id, media,
+        disable_notification=None, reply_to_message_id=None,
+        timeout=None, allow_sending_without_reply=None, protect_content=None):
+        method_url = r'sendMediaGroup'
+        media_json, files = Bot.convert_input_media_array(media)
+        dataset = {'chat_id': chat_id, 'media': media_json}
+        if disable_notification is not None:
+            dataset['disable_notification'] = disable_notification
+        if reply_to_message_id:
+            dataset['reply_to_message_id'] = reply_to_message_id
+        if timeout:
+            dataset['timeout'] = timeout
+        if allow_sending_without_reply is not None:
+            dataset['allow_sending_without_reply'] = allow_sending_without_reply
+        if protect_content is not None:
+            dataset['protect_content'] = protect_content
+        r = requests.post("https://api.telegram.org/bot{}/sendMediaGroup".format(self.token),params=dataset,files=files if files else None)
+        if "result" in r.text:
+            info = r.json()['result']
+            
+            return info
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    def promote(
+        self, chat_id, user_id, can_change_info=None, can_post_messages=None,
+        can_edit_messages=None, can_delete_messages=None, can_invite_users=None,
+        can_restrict_members=None, can_pin_messages=None, can_promote_members=None,
+        is_anonymous=None, can_manage_chat=None, can_manage_video_chats=None):
+        dataset = {'chat_id': chat_id, 'user_id': user_id}
+        if can_change_info is not None:
+            dataset['can_change_info'] = can_change_info
+        if can_post_messages is not None:
+            dataset['can_post_messages'] = can_post_messages
+        if can_edit_messages is not None:
+            dataset['can_edit_messages'] = can_edit_messages
+        if can_delete_messages is not None:
+            dataset['can_delete_messages'] = can_delete_messages
+        if can_invite_users is not None:
+            dataset['can_invite_users'] = can_invite_users
+        if can_restrict_members is not None:
+            dataset['can_restrict_members'] = can_restrict_members
+        if can_pin_messages is not None:
+            dataset['can_pin_messages'] = can_pin_messages
+        if can_promote_members is not None:
+            dataset['can_promote_members'] = can_promote_members
+        if is_anonymous is not None:
+            dataset['is_anonymous'] = is_anonymous
+        if can_manage_chat is not None:
+            dataset['can_manage_chat'] = can_manage_chat
+        if can_manage_video_chats is not None:
+            dataset['can_manage_video_chats'] = can_manage_video_chats
+        r =  requests.post("https://api.telegram.org/bot{}/promoteChatMember".format(self.token), params=dataset)
+    def get_member(self, chat_id, user_id):
+        r = requests.get("https://api.telegram.org/bot{}/getChatMember?chat_id={}&user_id={}".format(self.token,chat_id,user_id))
+        if "result" in r.text:
+            info = r.json()['result']
+            aa = {'userid':info['user']['id'],"fname":info['user']['first_name'],"username":info['user']['username']}
+            dict = dotdict(aa)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    def delete(self,chat_id,message_id):
+        r = requests.get("https://api.telegram.org/bot{}/deleteMessage?chat_id={}&message_id={}".format(self.token,chat_id,message_id))
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
     def edit_text(self,chat_id,message_id,text=None,parse_mode=None,reply_markup=None,disable_web_page_preview=None):
         if reply_markup == None:
             params = {
@@ -851,3 +962,182 @@ class Bot:
                 aa = {"badRequest":True,"message":des}
                 dict = dotdict(aa)
                 return dict
+    def answer(self, callback_query_id, text=None, show_alert=None, url=None, cache_time=None):
+        
+        dataset = {'callback_query_id': callback_query_id}
+        if text:
+            dataset['text'] = text
+        if show_alert is not None:
+            dataset['show_alert'] = show_alert
+        if url:
+            dataset['url'] = url
+        if cache_time is not None:
+            dataset['cache_time'] = cache_time
+        r = requests.post("https://api.telegram.org/bot{}/answerCallbackQuery".format(self.token), params=dataset)
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    def ban_chat(self, chat_id, sender_chat_id):
+        dataset = {'chat_id': chat_id, 'sender_chat_id': sender_chat_id}
+        r = requests.post("https://api.telegram.org/bot{}/banChatSenderChat".format(self.token), params=dataset)
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    def unban_chat(self, chat_id, sender_chat_id):
+        dataset = {'chat_id': chat_id, 'sender_chat_id': sender_chat_id}
+        r =  requests.post("https://api.telegram.org/bot{}/unbanChatSenderChat".format(self.token), params=dataset)
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    def admin_title(self, chat_id, user_id, custom_title):
+        
+        dataset = {
+            'chat_id': chat_id, 'user_id': user_id, 'custom_title': custom_title
+        }
+        r  =  requests.post("https://api.telegram.org/bot{}/setChatAdministratorCustomTitle".format(self.token), params=dataset)
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    def poll(
+        self, chat_id,
+        question, options,
+        is_anonymous = None, type = None, allows_multiple_answers = None, correct_option_id = None,
+        explanation = None, explanation_parse_mode=None, open_period = None, close_date = None, is_closed = None,
+        disable_notification=False, reply_to_message_id=None, allow_sending_without_reply=None,
+        reply_markup=None, timeout=None, explanation_entities=None, protect_content=None):
+        
+        dataset = {
+            'chat_id': str(chat_id),
+            'question': question,
+            'options': json.dumps(Bot._convert_poll_options(options))}
+
+        if is_anonymous is not None:
+            dataset['is_anonymous'] = is_anonymous
+        if type is not None:
+            dataset['type'] = type
+        if allows_multiple_answers is not None:
+            dataset['allows_multiple_answers'] = allows_multiple_answers
+        if correct_option_id is not None:
+            dataset['correct_option_id'] = correct_option_id
+        if explanation is not None:
+            dataset['explanation'] = explanation
+        if explanation_parse_mode is not None:
+            dataset['explanation_parse_mode'] = explanation_parse_mode
+        if open_period is not None:
+            dataset['open_period'] = open_period
+        if close_date is not None:
+            if isinstance(close_date, datetime):
+                dataset['close_date'] = close_date.timestamp()
+            else:
+                dataset['close_date'] = close_date
+        if is_closed is not None:
+            dataset['is_closed'] = is_closed
+
+        if disable_notification:
+            dataset['disable_notification'] = disable_notification
+        if reply_to_message_id is not None:
+            dataset['reply_to_message_id'] = reply_to_message_id
+        if allow_sending_without_reply is not None:
+            dataset['allow_sending_without_reply'] = allow_sending_without_reply
+        if reply_markup is not None:
+            dataset['reply_markup'] = Bot._convert_markup(reply_markup)
+        if timeout:
+            dataset['timeout'] = timeout
+        if explanation_entities:
+            dataset['explanation_entities'] = json.dumps(
+                regular.MessageEntity.to_list_of_dicts(explanation_entities))
+        if protect_content:
+            dataset['protect_content'] = protect_content
+        r = requests.get("https://api.telegram.org/bot{}/sendPoll".format(self.token,chat_id), params=dataset)
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
+    def _convert_markup(markup):
+        if isinstance(markup, regular.JsonSerializable):
+            return markup.to_json()
+        return markup
+    def _convert_poll_options(poll_options):
+        
+        if poll_options is None:
+            return None
+        elif len(poll_options) == 0:
+            return []
+        elif isinstance(poll_options[0], str):
+            # Compatibility mode with previous bug when only list of string was accepted as poll_options
+            return poll_options
+        elif isinstance(poll_options[0], regular.PollOption):
+            return [option.text for option in poll_options]
+        else:
+            return poll_options
+    def edit_capton(self, caption, chat_id=None, message_id=None, inline_message_id=None,
+                         parse_mode=None, caption_entities=None,reply_markup=None):
+        dataset = {'caption': caption}
+        if chat_id:
+            dataset['chat_id'] = chat_id
+        if message_id:
+            dataset['message_id'] = message_id
+        if inline_message_id:
+            dataset['inline_message_id'] = inline_message_id
+        if parse_mode:
+            dataset['parse_mode'] = parse_mode
+        if caption_entities:
+            dataset['caption_entities'] = json.dumps(regular.MessageEntity.to_list_of_dicts(caption_entities))
+        if reply_markup:
+            dataset['reply_markup'] = Bot._convert_markup(reply_markup)
+        r = requests.post("https://api.telegram.org/bot{}/editMessageCaption".format(self.token), params=dataset)
+    def convert_input_media(media):
+        if isinstance(media, regular.InputMedia):
+            return media.convert_input_media()
+        return None, None
+    def edit_media(self, media, chat_id=None, message_id=None, inline_message_id=None, reply_markup=None):
+        method_url = r'editMessageMedia'
+        media_json, file = Bot.convert_input_media(media)
+        dataset = {'media': media_json}
+        if chat_id:
+            dataset['chat_id'] = chat_id
+        if message_id:
+            dataset['message_id'] = message_id
+        if inline_message_id:
+            dataset['inline_message_id'] = inline_message_id
+        if reply_markup:
+            dataset['reply_markup'] = Bot._convert_markup(reply_markup)
+        r =  requests.post("https://api.telegram.org/bot{}/editMessageMedia".format(self.token), params=dataset, files=file)
+        if "result" in r.text:
+            info = r.json()['result']
+            dict = dotdict(info)
+            return dict
+        else:
+            des = r.json()['description']
+            aa = {"badRequest":True,"message":des}
+            dict = dotdict(aa)
+            return dict
\ No newline at end of file
